import pandas as pd
import requests



def get_data(series, fechaini, fechafin):

    ''' Importar multiples series de la API del BCRP.
    Considerar que las series deben ser de la misma frecuencia.
    
    Parámetros
    ----------
    series: dict
        Diccionario de los códigos y nombres de las series.
    fechaini: str
        Fecha de inicio de la serie.
    fechafin: str
        Fecha de fin de la serie.
        
    Retorno
    ----------
    df: pd.DataFrame
        Series consultadas.
    
    Ejemplo
    ----------
    Formatos para fechas:
    >>> Diario: yyyy-mm-dd
    >>> Mensual: yyyy-mm
    >>> Trimestral: yyyyQq
    >>> Anual: yyyy
    
    Observación
    ----------
    1. A veces las consultas a series 'trimestrales' extraen entre 1-4
       trimestres más o menos de los solicitados. Para evitar estos problemas
       de consulta con el servidor, consultar las 'fechaini' y 'fechafin' de
       inicio (Q1) al fin (Q4) de un rango temporal. Ejemplos:
           - 2000Q1-2022Q4
           - 1980Q1-2019Q4
           - etc...


    Documentación
    ----------
    https://estadisticas.bcrp.gob.pe/estadisticas/series/ayuda/api
    
    '''


    keys = list(series.keys())
    keysf = '-'.join(keys)    

    base = 'https://estadisticas.bcrp.gob.pe/estadisticas/series/api'

    url = f'{base}/{keysf}/json/{fechaini}/{fechafin}/ing'
    r = requests.get(url)
    
    try:
        response = r.json().get('periods')
    except ValueError:
        print('Error: Revisar que el formato de las fechas')
        print(' >>> Diario: yyyy-mm-dd\n >>> Mensual: yyyy-mm')
        print(' >>> Trimestral: yyyyQq\n >>> Anual: yyyy')
        return

    list_values = []
    list_time   = []

    for i in response:
        list_values.append(i['values'])    
        list_time.append(i['name'])

    df = pd.DataFrame(list_values)
    df = df.replace({'n.d.': None}) # Replace null values
    df = df.astype('float') # Set float format
    df.index  = list_time
    df.columns = list(series.values())


    # Formatos de fechas
    if keys[0][-1] == 'D':
        df.index = df.index.str.replace('Set', 'Sep')
        df.index = pd.to_datetime(df.index, format='%d.%b.%y')
    if keys[0][-1] == 'M':
        df.index = df.index.str.replace('Set', 'Sep')
        df.index = pd.to_datetime(df.index, format='%b.%Y')
    if keys[0][-1] == 'Q':
        try:
            df.index = df.index.map(lambda x: f"{x.split('.')[1]}{x.split('.')[0]}")
            new_fechaini = fechaini[0:2] + df.index[0]
            new_fechafin = fechafin[0:2] + df.index[-1]
            df.index = pd.period_range(new_fechaini, new_fechafin, freq='Q')
        except:
            pass
    if keys[0][-1] == 'A':
        df.index = pd.to_datetime(df.index, format='%Y').year
    
    df.sort_index(inplace=True) # Ordenamiento de fechas

    return df



def metadatos():

    metadatos = 'https://raw.githubusercontent.com/mauricioalvaradoo/'+\
                'econdata/master/src/econdata/metadata/BCRPData-metadata.csv'
    df = pd.read_csv(metadatos, index_col=0, sep=';', encoding='latin-1').reset_index()
    df = df[['Código de serie', 'Grupo de serie', 'Nombre de serie', 'Frecuencia', 'Fecha de inicio', 'Fecha de fin']]

    return df



def search(consulta, grupo=None, frecuencia=None):

    ''' Extraer código de la consulta.
    
    Parámetros
    ----------
    consulta: list
        Palabras claves de las series.
    grupo: list
        Palabras claves de los grupos. Default: None.
    frecuencia: str
        Frecuencia de la serie consultada. Default: None.
        >>> 'Diaria'
        >>> 'Mensual'
        >>> 'Trimestral'
        >>> 'Anual'

    Retorno
    ----------
    df: pd.DataFrame
        Metadatos de las series consultadas.
    
    Documentación
    ----------
    https://estadisticas.bcrp.gob.pe/estadisticas/series/ayuda/metadatos
    
    '''
    
    df = metadatos()
    consulta = [x.lower() for x in consulta]

    # Frecuencia
    if frecuencia is not None:
        try:
            df = df[df['Frecuencia'] == str(frecuencia)]
        except:
            pass
    
    # Grupo
    if grupo is not None:
        grupo = [x.lower() for x in grupo]
        
        for i in grupo:
            try:
                filter = df['Grupo de serie'].str.lower().str.contains(i)
                df = df[filter]
            except:
                pass

    # Series
    for i in consulta:           
        try:
            filter = df['Nombre de serie'].str.lower().str.contains(i)
            df = df[filter]
        except:
            df = print('Consulta no encontrada!')
    
    df.set_index('Código de serie', inplace=True)
    return df



def documentation(code):
    
    ''' Extraer microdatos del código de la serie.
    
    Parámetros
    ----------
    code: str
        Código base de la serie.

    Retorno
    ----------
    df: pd.DataFrame
        Metadatos de las series consultadas.
      
    Documentación
    ----------
    https://estadisticas.bcrp.gob.pe/estadisticas/series/ayuda/metadatos
    
    '''

    df = metadatos()
    df = df[df['Código de serie'] == str(code)]

    return df





import pandas as pd
import requests



def get_data(indicator, countries=None, fechaini=None, fechafin=None):
    
    ''' Importar series de la API del FMI.
    
    Parámetros
    ----------
    indicator: str
        Código del indicador o serie.
        Se obtiene de: 'https://www.imf.org/external/datamapper/api/v1/'       
    countries: dict
        Diccionario de los códigos y nombres de los países. Default: None.
        >>> countries = {'PER': 'Perú'}   
    fechaini: str
        Fecha de inicio de la serie. Default: None.
        >>> Anual: yyy
    fechafin: str
        Fecha de fin de la serie. Default: None
        >>> Anual: yyyy
        
    Retorno
    ----------
    df: pd.DataFrame
        Países con las series consultadas.    
    
    Documentación
    ----------
    https://www.imf.org/external/datamapper/api/help

    '''
    
    base = 'https://www.imf.org/external/datamapper/api/v1/'
    
    # Filtros
    filters = '/'.join(countries.keys()) if countries is not None else None
    dates = [str(y) for y in range(int(fechaini), int(fechafin)+1)] if fechaini is not None and fechafin is not None else None
    dates = [str(y) for y in range(int(fechaini), 2029)]            if fechaini is not None and fechafin is     None else dates
    dates = [str(y) for y in range(1980, int(fechafin)+1)]          if fechaini is     None and fechafin is not None else dates
    dates = ','.join(dates) if dates is not None else None
    
    url = f'{base}{indicator}'
    
    if filters:
        url += f'/{filters}'      
    if dates:
        url += f'?periods={dates}'
    
    try:
        serie = requests.get(url).json()['values'][indicator]
        df = pd.DataFrame(serie)
        df.sort_index(inplace=True)
    except KeyError:
        return print(
            'El código de la serie o del país es incorrecto.',
            'En el caso del país, confirmar si está incluido en la base de datos del que pertenece la serie.')
        
    return df



def search(tipo, consulta=None):
    
    ''' Extraer código de la consulta.
    
    Parámetros
    ----------
    tipo: str
        Tipo de datos a consultar: 'Indicadores', 'Países', 'Regiones', 'Grupos'.
    consulta: list
        Palabras claves de consulta. Default: None.

    Retorno
    ----------
    df: pd.DataFrame
        Consulta.

    Documentación
    ----------
    * https://www.imf.org/en/Data
    * https://www.imf.org/external/datamapper/api/help
    
    '''
    
    base = 'https://www.imf.org/external/datamapper/api/v1'
    
    def search_df1(r):
        codes = []; names = []; units = []; datasets = []
        for i in list(r.keys()):
            codes.append(i)
        for i in list(r.values()):
            names.append(i['label'])
            units.append(i['unit'])
            datasets.append(i['dataset'])
        df1 = pd.DataFrame({'Código':codes,'Nombres':names,'Unidades':units,'Dataset':datasets})
    
        return df1
    
    def search_df2(r):
        codes = []; names = []
        for i in list(r.keys()):
            codes.append(i)
        for i in list(r.values()):
            names.append(i['label'])
        df2 = pd.DataFrame({'Código': codes, 'Nombres': names})
    
        return df2
    
    
    types = {
        'Indicadores': 'indicators',
        'Países': 'countries',
        'Regiones': 'regions',
        'Grupos': 'groups'
    }
    
    
    try:
        selection = types[tipo]
        url = f'{base}/{selection}'
        r = requests.get(url).json()[selection]
        
        try: 
            df = search_df1(r)
        except:
            df = search_df2(r)
        
        df.dropna(inplace=True)

        if consulta is not None:
            consulta = [x.lower() for x in consulta]
                
            for i in consulta:
                filter = df['Nombres'].str.lower().str.contains(i)
                df = df[filter]

        else:
            pass
    
        df.set_index('Código', inplace=True)
    
        return df     
        
    except:
        return print('Revisa bien el tipo!')


import pandas as pd
import yfinance as yf



def get_data(series, tipo='Close', fechaini=None, fechafin=None):

    ''' Importar multiples series de la API de Yahoo Finance.
    
    Parámetros
    ----------
    series: dict
        Lista de los codigos de las series y nombres.
    tipo: str
        Serie consultada.
        >>> Close: Default
        >>> Open
        >>> High
        >>> Low
        >>> AdjClose
        >>> Volume
    fechaini: str
        Fecha de inicio de la serie.
        >>> Diario: yyyy-mm-dd
    fechafin: str
        Fecha de fin de la serie.
        >>> Diario: yyyy-mm-dd

    Retorno
    ----------
    df: pd.DataFrame
        Series consultadas.
    
    Documentación
    ----------
    https://pypi.org/project/yfinance/
    
    '''


    keys = list(series.keys())
    df = pd.DataFrame()
    
    for key in keys:
        data = yf.Ticker(key)
        
        if (fechaini == None) or (fechafin == None):
            data = data.history(period='max')
        else:
            try:
                data = data.history(start=fechaini, end=fechafin)
            except ValueError:
                print('Error: Revisar que el formato de las fechas')
                print(' >>> Diario: yyyy-mm-dd')
                return
                
        data.reset_index(inplace=True)
        data[key] = data[tipo]

        try: 
            # Quitar zona horaria
            data['Date'] = data['Date'].dt.tz_localize(None)
        except:
            pass
    
        # Merge
        df = pd.concat([df, data[['Date', key]]]) if df.empty is True else pd.merge(df, data[['Date', key]], how='outer')

    df.set_index('Date', inplace=True)
    df.index = pd.to_datetime(df.index, format='%Y-%m-%d')
    df.sort_index(inplace=True)
    df = df.astype('float')
    df = df.rename(series, axis = 1)


    return df



def search(consulta):
    
    ''' Extraer código de la consulta.
    
    Parámetros
    ----------
    consulta: list
        Palabras claves de las series.

    Retorno
    ----------
    df: pd.DataFrame
        Metadatos de las series consultadas.
    
    Documentación
    ----------
    https://www.nasdaq.com/market-activity/stocks/screener
    
    '''
    
    df = pd.read_csv(
        'https://raw.githubusercontent.com/mauricioalvaradoo/econdata/master/src/econdata/metadata/Yahoo-Tickers.csv',
        index_col=0
    ).reset_index()
    df = df[['Symbol', 'Name', 'Country', 'IPO Year', 'Sector', 'Industry']]
    df.set_index('Symbol', inplace=True)
    
    consulta = [x.lower() for x in consulta]
    
    for i in consulta:           
        try:
            filter = df['Name'].str.lower().str.contains(i)
            df = df[filter]
        except:
            df = print('Consulta no encontrada!')
            return
        
    try:
        df['IPO Year'] = df['IPO Year'].astype('Int64')
    except:
        pass
    
    return df






import pandas as pd
import requests 



def get_data(identifier, countries, serie, fechaini, fechafin, periodicidad):
    
    ''' Importar multiples series de la API de la OCDE.
    
    Parámetros
    ----------
    identifier: str
        Código identificador de la base de datos.
    countries: dict
        Códigos y nombres de los países (keys, values).
    serie: str
        Código de la serie.
    fechaini: str
        Fecha de inicio (yyyy-qq) .
    fechafin: str
        Fecha de fin (yyyy-qq).
    periodicidad: str
        Frecuencia de los datos: 'Q', 'Y'.  
    
    Retorno
    -------
    df: pd.DataFrame
        Series consultadas.
    
    Documentación
    -------
    https://stats.oecd.org/
    
    '''

    base = 'https://stats.oecd.org/SDMX-JSON'
    filters = '+'.join(countries.keys())
    nombres = list(countries.values())
    cantidad_paises = len(countries.keys())
    
    url = f'{base}/data/{identifier}/{filters}.{serie}/all?startTime={fechaini}&endTime={fechafin}'

    r = requests.get(url)
    if r.status_code == 200:
        pass
    else:
        print('Vinculacion inválida!')
    series = r.json()['dataSets'][0]['series']    
    
    df = pd.DataFrame()
    j = 0

    for i in range(0, cantidad_paises):
        fechas = list(list(series.values())[i]['observations'].keys())    # Índices con fechas
        valores = list(list(series.values())[i]['observations'].values()) # Valores en bruto
        
        # Extraer los valores limpios
        nv = []
        for i in range(0, len(fechas)):
            nv.append(valores[i][0])

        if df.empty == True:
            df = pd.DataFrame({'fechas': fechas, nombres[j]: nv})
        else:
            df2 = pd.DataFrame({'fechas': fechas, nombres[j]: nv})
            df = df.merge(df2, how='inner')
        j += 1    
      
    df = df.set_index('fechas')
    df.index = pd.period_range(fechaini, fechafin, freq=periodicidad)
    df.sort_index(inplace=True)
    df = df.astype('float')
    
    return df


import setuptools

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name='econdata',
    version = '1.0.16',
    author = 'Mauricio Alvarado',
    description = 'Extracción de series de tiempo de las principales instituciones económicas para el Perú',
    long_description = long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/mauricioalvaradoo/econdata',
    classifiers = [
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent'
    ], 
    package_dir={'':'src'},
    # packages=setuptools.find_packages(),
    python_requires='>=3.6',
    install_requires = [
        'pandas',
        'numpy',
        'yfinance',
        'requests',
        'warn',
        'more-itertools'
    ]
)


import pandas as pd
import requests
import itertools



def get_data(countries, indicators, fechaini, fechafin):
    
    ''' Importar múltiples series de la API del Banco Mundial.
    
    Parámetros
    ----------
    countries: dict
        Diccionario de los códigos y nombres de los países.
        >>>  countries = {'PE': 'Perú'}
    indicators: dict
        Diccionario de los códigos de los indicadores o series.
        Se obtienen de: 'https://datos.bancomundial.org/indicator/'
        >>> indicators = {'NY.GDP.PCAP.PP.KD': 'GDP per capita, PPP (US$ 2017)'}
    fechaini: str
        Fecha de inicio de la serie.
        >>> Anual: yyyy
    fechafin: str
        Fecha de fin de la serie.
        >>> Anual: yyyy
        
    Retorno
    ----------
    df: pd.DataFrame
        Países con las series consultadas.    
    
    Documentación
    ----------
    https://datahelpdesk.worldbank.org/knowledgebase/articles/898581-api-basic-call-structure

    '''


    df = pd.DataFrame()
        
    
    for c_k, c_n in countries.items(): 
        for i_k, i_n in indicators.items():
                
            url = f'http://api.worldbank.org/v2/country/{c_k}/indicator/{i_k}?format=json'
                 
            r = requests.get(url) 
            if r.status_code == 200:
                pass
            else:
                print('Vinculacion inválida!')          
            response = r.json()[1]
            
            values_list = []
            time_list   = []
                
            for j in response: 
                values_list.append(j['value'])
                time_list.append(j['date'])
                
            # Formats
            dictio = pd.DataFrame({'time': time_list, f'{i_k}': values_list})
            dictio.set_index('time', inplace = True)
            dictio.sort_index(ascending=True, inplace=True)
            dictio.rename(indicators, axis=1, inplace=True)
            
            # MultiIndex
            dictio.columns = pd.MultiIndex.from_tuples([(c_n, i_n)])
            
            # Merge
            df = pd.concat([df, dictio], axis=1)

    df = df.loc[fechaini: fechafin]
    df = df.astype('float')
    
    return df




def search(consulta):
        
    ''' Extraer metadatos.
    
    Parámetros
    ----------
    consulta: list
        Dos palabras clave de la consulta.
        También se puede consultar en: https://datos.bancomundial.org/indicator/
        
    Retorno: 
    ---------
    df: pd.DataFrame
       Series consultadas.

    '''


    consulta = [w.lower() for w in consulta]
    
    # Combinations of words
    combs = list(itertools.permutations(consulta))
    combinations = []
    for co in combs:
        combination = '%20'.join(co)
        combinations.append(combination)
    
    # Query
    list_id    = []
    list_names = []
    
    for f in combinations:
        try:
            url = f'http://api.worldbank.org/v2/sources/2/search/{f}?format=json'
            r = requests.get(url) 
            response = r.json()['source'][0]['concept']    
        
            for res in response:
                try:
                    ii = res['variable']
                    for i in ii:
                        list_id.append(i['id'])
                        list_names.append(i['metatype'][0]['value'])
                except:
                    pass
        except:
            pass

    df = pd.DataFrame({'id': list_id, 'title': list_names})
    df.set_index('id', inplace=True)
    df = df.drop_duplicates() # Drop duplicated searchs
    df = df[~df.index.str.contains('~')]
    
    
    if df.empty is True:
        print('No encontrado!')
    else:
        return df


import pandas as pd
import numpy as np
import requests 
import warnings
warnings.simplefilter('ignore')



def get_data(series, api_key, fechaini, fechafin):
    
    ''' Importar múltiples series de la API del FRED.
    
    Parámetros
    ----------
    series: dict
        Diccionario de los códigos y nombres de las series.
    api_key: str
        API Key del desarrollador: https://fred.stlouisfed.org/docs/api/api_key.html
    fechaini: str
        Fecha de inicio de la serie.
        >>> Diario: yyyy-mm-dd
        >>> Mensual: yyyy-mm
        >>> Trimestral: yyyyQq
        >>> Anual: yyyy
    fechafin: str
        >>> Diario: yyyy-mm-dd
        >>> Mensual: yyyy-mm
        >>> Trimestral: yyyyQq
        >>> Anual: yyyy
 
    Retorno 
    ----------
    df: pd.DataFrame
       Series consultadas.
        
    Documentación
    ----------
    https://fred.stlouisfed.org/docs/api/fred/
    
    '''
      
    
    keys = list(series.keys())

    df = pd.DataFrame()
    
    for i in keys:
        url = f'https://api.stlouisfed.org/fred/series/observations?series_id={i}&api_key={api_key}&file_type=json'
    
        r = requests.get(url)
        if r.status_code == 200:
            pass
        else:
            print('Vinculacion inválida!')
        observations = r.json().get('observations')
    
        list_values = []
        list_time = []
        
        for j in observations:
            list_values.append(j['value'])
            list_time.append(j['date'])


        dictio = pd.DataFrame({'time': list_time, f'{i}': list_values})
        df = dictio if df.empty is True else pd.merge(df, dictio, how='outer')

    df.set_index('time', inplace=True)
    df.rename(series, axis = 1, inplace = True)
    
    # Corrigiendo '.' for missing values   
    for i in df.columns:
        try:
            df.loc[df[i] == '.', i] = np.nan
            df[i] = df[i].astype('float')
        except:
            pass

    # index to datetime
    df.index = pd.to_datetime(df.index)
    df.sort_index(inplace=True)
    df = df[(df.index>fechaini)&(df.index<fechafin)]

    return df




def search(consulta, api_key, frecuencia=None):
    
    ''' Extraer metadatos de las .

    Parámetros
    ----------
    consulta: list
        Palabras claves de las consultas.
    api_key: str
        API Key del desarrollador: https://fred.stlouisfed.org/docs/api/api_key.html
    frecuencia: str
        Frecuencia de la serie consultada. Default: None.
        >>> "Daily"
        >>> "Monthly"
        >>> "Quarterly"
        >>> "Annual"
            
    Retorno
    ----------
    df: pd.DataFrame
       Series consultadas.
    
    '''
    
    formato = '+'.join(consulta)
    formato = formato.lower()

    url = f'https://api.stlouisfed.org/fred/series/search?search_text={formato}&api_key={api_key}&file_type=json'
    r = requests.get(url) 

    response = r.json()['seriess']

    list_id = []
    list_title = []
    list_start = []
    list_end = []
    list_frequency = []
    list_seasonal_adjusment = []

    for i in response:
        list_id.append(i['id'])
        list_title.append(i['title'])
        list_start.append(i['observation_start'])
        list_end.append(i['observation_end'])
        list_frequency.append(i['frequency'])
        list_seasonal_adjusment.append(i['seasonal_adjustment'])

    df = pd.DataFrame({
        'id': list_id,
        'title': list_title,
        'start': list_start,
        'end': list_end,
        'frequency': list_frequency,
        'seasonal_adjusment': list_seasonal_adjusment
    })

    if frecuencia is not None:
        try:
            df = df[df['frequency'] == str(frecuencia)]
        except:
            pass

    df.set_index('id', inplace=True) 

    return df

